class NewMPQuery(object):
    # Chris B key = E4p0iuBO8rX7XTaPe5c4O8WO5YpikMB1

    """
    Trying to transition to new API but running into many issues
    """

    def __init__(self, api_key):
        """
        Args:
            api_key (str) - Materials Project API key

        Returns:
            self.mpr (MPRester) - Materials Project REST interface
        """

        api_key = api_key if api_key else "YOUR_API_KEY"

        self.api_key = api_key
        self.mpr = new_MPRester(api_key)

    @property
    def available_fields(self):
        return self.mpr.summary.available_fields

    @property
    def typical_fields(self):
        return [
            "formula_pretty",
            "volume",
            "nsites",
            "material_id",
            "energy_per_atom",
            "uncorrected_energy_per_atom",
            "formation_energy_per_atom",
            "energy_above_hull",
            "equilibrium_reaction_energy_per_atom",
            "decomposes_to",
            "band_gap",
            "is_magnetic",
            "symmetry",
        ]

    def get_data_for_comp(self, comp):
        all_chemsyses = None
        all_formulas = None
        if isinstance(comp, str):
            if "-" in comp:
                chemsys = comp
                all_chemsyses = []
                elements = chemsys.split("-")
                for i in range(len(elements)):
                    for els in itertools.combinations(elements, i + 1):
                        all_chemsyses.append("-".join(sorted(els)))
            else:
                all_formulas = [CompTools(comp).clean]
        elif isinstance(comp, list):
            if "-" in comp[0]:
                all_chemsyses = []
                for chemsys in comp:
                    elements = chemsys.split("-")
                    for i in range(len(elements)):
                        for els in itertools.combinations(elements, i + 1):
                            all_chemsyses.append("-".join(sorted(els)))
                all_chemsyses = sorted(list(set(all_chemsyses)))
            else:
                all_formulas = [CompTools(c).pretty for c in comp]
        if all_formulas:
            from_mp = self.mpr.summary.search(
                formula_pretty=all_formulas, fields=self.typical_fields
            )
        elif all_chemsyses:
            from_mp = self.mpr.summary.search(
                chemsys=all_chemsyses, fields=self.typical_fields
            )

        return from_mp



import os
from pydmclab.hpc.helpers import (
    get_query,
    check_query,
    get_strucs,
    check_strucs,
    get_magmoms,
    check_magmoms,
    get_launch_dirs,
    check_launch_dirs,
    submit_calcs,
    get_results,
    check_results,
    get_gs,
    check_gs,
    get_thermo_results,
    check_thermo_results,
    get_launch_configs,
    get_sub_configs,
    get_slurm_configs,
    get_vasp_configs,
    get_analysis_configs,
    make_sub_for_launcher,
)

# where is this file
SCRIPTS_DIR = os.getcwd()

# where are my calculations going to live (maybe on scratch)
CALCS_DIR = SCRIPTS_DIR.replace("scripts", "calcs")

# where is my data going to live
DATA_DIR = SCRIPTS_DIR.replace("scripts", "data")

for d in [CALCS_DIR, DATA_DIR]:
    if not os.path.exists(d):
        os.makedirs(d)

# if you need data from MP as a starting point (often the case), you need your API key
API_KEY = "__YOUR API KEY__"

# what to query MP for (if you need MP data)
## e.g., 'MnO2', ['MnO2', 'TiO2'], 'Ca-Ti-O, etc
COMPOSITIONS = None

# any configurations related to LaunchTools
LAUNCH_CONFIGS = get_launch_configs(
    standards=["dmc"],
    xcs=["metagga"],
    use_mp_thermo_data=False,
    n_afm_configs=0,
    skip_xcs_for_standards={"mp": ["gga", "metagga"]},
)

# any configurations related to SubmitTools
SUB_CONFIGS = get_sub_configs(
    submit_calculations_in_parallel=False,
    rerun_lobster=False,
    mpi_command="mpirun",
    special_packing=False,
    delete_all_calculations_and_start_over=False,
    machine="msi",
)

# any configurations related to Slurm
SLURM_CONFIGS = get_slurm_configs(
    total_nodes=1,
    cores_per_node=8,
    walltime_in_hours=95,
    mem_per_core="all",
    partition="agsmall,msidmc",
    error_file="log.e",
    output_file="log.o",
    account="cbartel",
)

# any configurations related to VASPSetUp
VASP_CONFIGS = get_vasp_configs(
    run_lobster=False,
    modify_loose_incar=False,
    modify_relax_incar=False,
    modify_static_incar=False,
    modify_loose_kpoints=False,
    modify_relax_kpoints=False,
    modify_static_kpoints=False,
    modify_loose_potcar=False,
    modify_relax_potcar=False,
    modify_static_potcar=False,
)

# any configurations related to AnalyzeBatch
ANALYSIS_CONFIGS = get_analysis_configs(
    analyze_calculations_in_parallel=False,
    analyze_structure=True,
    analyze_mag=False,
    analyze_charge=False,
    analyze_dos=False,
    analyze_bonding=False,
    exclude=[],
)

# whether or not you want to generate MAGMOMs (only if you're running AFM)
GEN_MAGMOMS = True if LAUNCH_CONFIGS["n_afm_configs"] else False

# NOTE: the default is to use the imported functions from pydmclab.hpc.helpers
# You will often want to write your own "get_query" and/or "get_strucs" functions instead
# See below (or within pydmclab.hpc.helpers) for some more detailed docs


def main():
    remake_sub_for_launcher = False

    remake_query = False
    print_query_check = True

    remake_strucs = False
    print_strucs_check = True

    remake_magmoms = False
    print_magmoms_check = True

    remake_launch_dirs = False
    print_launch_dirs_check = True

    remake_subs = True
    ready_to_launch = True

    remake_results = True
    print_results_check = True

    remake_gs = True
    print_gs_check = True

    remake_thermo_results = True
    print_thermo_results_check = True

    if remake_sub_for_launcher:
        make_sub_for_launcher()

    comp = COMPOSITIONS
    query = get_query(
        comp=comp, api_key=API_KEY, data_dir=DATA_DIR, remake=remake_query
    )
    if print_query_check:
        check_query(query)

    strucs = get_strucs(query=query, data_dir=DATA_DIR, remake=remake_strucs)
    if print_strucs_check:
        check_strucs(strucs)

    if GEN_MAGMOMS:
        magmoms = get_magmoms(strucs=strucs, data_dir=DATA_DIR, remake=remake_magmoms)
        if print_magmoms_check:
            check_magmoms(strucs=strucs, magmoms=magmoms)
    else:
        magmoms = None

    launch_configs = LAUNCH_CONFIGS
    launch_dirs = get_launch_dirs(
        strucs=strucs,
        magmoms=magmoms,
        user_configs=launch_configs,
        data_dir=DATA_DIR,
        remake=remake_launch_dirs,
    )
    if print_launch_dirs_check:
        check_launch_dirs(launch_dirs)

    sub_configs = SUB_CONFIGS
    slurm_configs = SLURM_CONFIGS
    vasp_configs = VASP_CONFIGS
    user_sub_configs = {**sub_configs, **slurm_configs, **vasp_configs}
    if remake_subs:
        submit_calcs(
            launch_dirs=launch_dirs,
            calcs_dir=CALCS_DIR,
            user_configs=user_sub_configs,
            ready_to_launch=ready_to_launch,
            n_procs=sub_configs["n_procs"],
        )

    analysis_configs = ANALYSIS_CONFIGS
    results = get_results(
        launch_dirs=launch_dirs,
        user_configs=analysis_configs,
        data_dir=DATA_DIR,
        remake=remake_results,
    )
    if print_results_check:
        check_results(results)

    gs = get_gs(results=results, data_dir=DATA_DIR, remake=remake_gs)

    if print_gs_check:
        check_gs(gs)

    thermo = get_thermo_results(
        results=results, gs=gs, data_dir=DATA_DIR, remake=remake_thermo_results
    )

    if print_thermo_results_check:
        check_thermo_results(thermo)

    return


if __name__ == "__main__":
    main()

    """
    see [pydmclab docs](https://github.umn.edu/bartel-group/pydmclab/blob/main/docs.md) for help

    The basic idea:

    1. get_query
        - need some data to start with that has at least one crystal structure
        - the imported `get_query` will use pydmclab.core.query.MPQuery to get MP data
        - alternatively, you may want to use other starting points. e.g.,
            - other calculations you've run
            - experimental data

        Must return:
            {ID (str) : {'structure' : Pymatgen Structure as dict,
                        < any other data you want to keep track of >}}

        Notes:
            - ID in this case must be a standalone identifier for that structure
                - e.g., mp-123456

    2. get_strucs
        - now we're going to convert our query into the POSCARs we want to calculate
        - a "structure" corresponds with a chemical composition (formula) and a unique identifier for that formula (ID)
        - the imported `get_strucs` will only work if you are not transforming the structures present in your query
        - you will often want to write your own version of this. e.g.,
            - to replace species, change occupancies, etc from the structures you "queried" for

        Must return:
            {formula identifier (str) :
                {structure identifier for that formula (str) :
                    Pymatgen Structure object as dict}}

        Notes:
            - formula identifier in this case must be a standalone identifier for that formula
            - ID only has to be unique within each formula
                - e.g., if you calculate LiMnO2 with many different Li/Mn orderings, ID could be 0, 1, 2, ... for each ordering

    3. get_magmoms
        - if we're running AFM calculations, this will get us our magmoms for each AFM ordering of each structure
        - you can generally use the imported version of this function

        Must return:
            {formula identifier (str) :
                {structure identifier for that formula (str) :
                    {AFM ordering identifier (str) :
                        [list of magmoms (floats) for each site in the structure]}}}

    4. get_launch_dirs
        - this will get us the directories we want to run our calculations in
        - you can generally use the imported version of this function
        - use LAUNCH_CONFIGS to specify what calculations get launched

        Must return:
            {launch dir (str, formula/ID/stsandard/mag) :
                {'xcs' : [list of final xcs to run (str)],
                'magmoms' : [list of magmoms (floats) for each site in the structure]}}

    5. submit_calcs
        - this will write our submission scripts and submit our calculations
        - you can generally use the imported version of this function
        - use SUB_CONFIGS, SLURM_CONFIGS, VASP_CONFIGS to modify how these calculations are prepared

        Does not return anything

    6. get_results
        - this will collect all of the results we specified to collect
        - you can generally use the imported version of this function
        - use ANALYSIS_CONFIGS to modify what results are collected

        Must return:
            {formula--ID--standard--mag--xc-calc (str) :
                {scraped results from VASP calculation}}

    7. get_gs
        - this will collect the ground-state (lowest energy) result for each calculated formula
        - you can generally use the imported version of this function

        Must return:
            {standard (str) :
                {xc (str) :
                    {formula (str) :
                        {'E' : energy of the ground-structure,
                        'key' : formula.ID.standard.mag.xc_calc for the ground-state structure,
                        'structure' : structure of the ground-state structure,
                        'n_started' : how many polymorphs you tried to calculate,
                        'n_converged' : how many polymorphs are converged,
                        'complete' : True if n_converged = n_started (i.e., all structures for this formula at this xc are done),
                        'Ef' : the formation enthalpy (0 K)}

    8. get_thermo_results
        - this will collect the thermodynamic results for all calculated structures (including non-ground-states)
        - you can generally use the imported version of this function

        Must return:
            {standard (str) :
                {xc (str) :
                    {formula (str) :
                        {ID (str) :
                            {'E' : energy of the structure (DFT total energy in eV/atom),
                            'Ef' : formation enthalpy at 0 K (eV/atom),
                            'is_gs' : True if this is the lowest energy polymorph for this formula,
                            'dE_gs' : how high above the ground-state this structure is in energy (eV/atom)
                            'all_polymorphs_converged' : True if every structure that was computed for this formula is converged}}
    """



from pydmclab.core.struc import StrucTools, SiteTools
from pydmclab.core.query import MPQuery

from pymatgen.core.structure import Structure


""" 
Purpose: 
  StrucTools is used to perform manipulations to crystal structures
  SiteTools helps you do this by extracting information for individual sites in a structure
  
Unit tests:
  - see pydmclab/tests/test_struc.py

"""

MPID = "mp-19417"
API_KEY = "N3KdATtMmcsUL94g"


def demo_basic_structure_manipulations(mpid=MPID):
    """
    - get the structure formula
    - reduce that formula (get the clean version)
    - get the elements in the structure
    - count the number of sites
    - make a supercell
    """

    print("\n ~~ demoing basic manipulations ~~ \n")

    s = MPQuery(API_KEY).get_structure_by_material_id(mpid)
    st = StrucTools(s)

    print("\nformula = %s" % st.formula)
    print("compact formula = %s" % st.compact_formula)
    print("structure has %s" % st.els)
    print("downloaded structure has %i sites" % len(st.structure))
    st.make_supercell([1, 2, 3])
    print("supercell has %i sites" % len(st.structure))


def demo_ox_state_decoration(mpid=MPID):
    """
    - decorate w/ oxidation states using pymatgen's automatic algorithm
    - decorate w/ user-specified oxidation states
    - inspect the oxidation state using SiteTools.ion
    """
    print("\n ~~ demoing oxidation state decoration ~~ \n")
    s = MPQuery(API_KEY).get_structure_by_material_id(mpid)
    st = StrucTools(s)

    site = SiteTools(s, 0)
    print("from MP, site 0 is %s" % site.ion)

    s = st.decorate_with_ox_states

    print("after auto oxi state decoration, site 0 is %s" % SiteTools(s, 0).ion)

    st = StrucTools(s, ox_states={"Fe": 3, "Ti": 3, "O": -2})

    s = st.decorate_with_ox_states

    print(
        "after forcing Fe and Ti to be ox state = 3, site 0 is %s" % SiteTools(s, 0).ion
    )


def demo_replace_species_and_order(mpid=MPID):
    """
    - make a supercell
    - replace a fullly occupied site with a mixed occupancy site
    - generate ordered versions of this mixed occupancy structure
    - inspect some sites in one of the ordered structures
    """
    print("\n ~~ demoing replace species ~~ \n")
    s = MPQuery(API_KEY).get_structure_by_material_id(mpid)
    st = StrucTools(s)
    st.make_supercell([1, 2, 3])

    species_map = {"Fe": {"Fe": 0.875, "Cr": 0.125}}

    ordered_strucs = st.replace_species(species_map, n_strucs=10)

    print("generated %i structures" % len(ordered_strucs))

    struc_idx = 3
    struc = ordered_strucs[struc_idx]
    print("\nstructure %i has formula %s" % (struc_idx, StrucTools(struc).formula))
    site_idx = 12
    site = SiteTools(struc, site_idx)
    print(
        "site %i has el = %s, coords = %s with ox state = %s"
        % (site_idx, site.el, site.coords, site.ox_state)
    )


def demo_dilute_vacancy(mpid=MPID):
    """
    - change the occupancy of an element from 1 (fully occupied) to some value that leads to 1 vacancy

    """
    print("\n ~~ demoing dilute vacancy ~~ \n")
    s = MPQuery(API_KEY).get_structure_by_material_id(mpid)
    st = StrucTools(s)

    print("initial formula = ", st.formula)

    el_to_remove = "O"
    n_el_in_struc = st.amts["O"]
    n_el_to_remove = 1
    new_occ = (n_el_in_struc - n_el_to_remove) / n_el_in_struc

    s_with_vac = st.change_occ_for_el(
        el_to_remove, {el_to_remove: new_occ}, structure=None
    )

    ordered_strucs = StrucTools(
        s_with_vac, ox_states={"O": -2, "Fe": 3, "Ti": 3}
    ).get_ordered_structures(n_strucs=10)

    print("final formula = ", StrucTools(ordered_strucs[0]).formula)


def main():
    demo_basic_structure_manipulations()
    demo_ox_state_decoration(mpid=MPID)
    demo_replace_species_and_order(mpid=MPID)
    demo_dilute_vacancy(mpid=MPID)
    return


if __name__ == "__main__":
    main()



import matplotlib as mpl
import matplotlib.pyplot as plt

import numpy as np

from pydmclab.plotting.utils import set_rc_params, get_colors
from pydmclab.data.plotting_configs import get_color_palettes
from scipy.ndimage import gaussian_filter1d

import random


# Please note that this function only deals with the case of total spin
def ax_tdos(
    tdos,
    population_sources="all",
    colors={"total": "black"},
    color_palette=get_colors("set2"),
    params={"line_alpha": 0.9, "fill_alpha": 0.2, "lw": 1},
    special_labels=None,
    spins="summed",
    normalization=None,
    smearing=0.2,
    Efermi=0.0,
    xlim=(0, 10),
    ylim=(-2, 2),
    xticks=(True, [0, 5, 10]),
    yticks=(True, [-2, -1, 0, 1, 2]),
    xlabel="DOS",
    ylabel=r"$E-E_F\/(eV)$",
    legend=True,
    title=None,
    savename=False,
    show=True,
):
    """
    Args:
        tdos (dict)
            result of pydmclab.hpc.analyze.tdos()
                list(d.keys()) = ['E', 'total', 'up', 'down'] + [list of elements (str)]
                    d['E'] = 1d array of energies corresponding with DOS
                    d[el] = 1d array of DOS for that element (sums all sites, orbitals, and spins)
                    d['total'] = 1d array of DOS for structure (sums all elements, sites, orbitals, spins)
                    d['up'] or d['down']:
                        keys are ['total'] + [list of elements (str)]
                        d['up']['total'] = 1d array of spin-up DOS for structure
                        d['down'][el] = 1d array of spin-down DOS for that element
                        etc
                so if I wanted to plot the total DOS for my structure and separate spin up (+ DOS) and spin down (- DOS)
                    energies = d['E']
                    dos_up = d['up']['total']
                    dos_down = d['down']['total']
                    plt.plot(dos_up, energies)
                    plt.plot(-1*dos_down, energies)

        population_sources (str or list)
            'all' : plot all elements and the total
            ['total'] : plots just the total
            ['Al', 'N'] : plots just Al and N
            ['Al'] : plots just Al

        colors (dict)
            {element or 'total' (str) : color (str)}

        color_palette (dict)
            {color (str) : rgb (tuple)}
                usually loaded from pydmc.utils.plotting.get_colors()

        params (dict)
            {'fill_alpha' : transparency for occ populations,
             'line_alpha' : transparency for DOS line,
             'lw' : DOS linewidth}

        special_labels (dict)
            {element or 'total' (str) : label (str)}

        spins (str)
            'summed' : plot + and - DOS together as +
            'separate' : plot + and - DOS separately as + and -
                @TO-DO: implement separate spins

        normalization (float)
            divide DOS by this number
                common normalizations:
                    1 : no normalization (same as None)
                    CompTools(formula).n_atoms (per formula unit)
                    results['meta']['all_input_parameters']['NELECT'] (per electron)

        smearing (float or False)
            std. dev. for Gaussian smearing of DOS or False for no smearing

        Efermi (float)
            Fermi level (eV)

        xlim (tuple)
            (xmin (float), xmax (float))

        ylim (tuple)
            (ymin (float), ymax (float))

        xticks (tuple)
            (bool to show label or not, (xtick0, xtick1, ...))

        xlabel (str)
            x-axis label

        ylabel (str)
            y-axis label

        legend (bool)
            include legend

        title (str)
            title of plot

        savename (str)
            if False: don't save (just return ax)
            if str, save fig object at this location

        show (bool)
            show plot

    Returns:
        matplotlib axis object
    """
    set_rc_params()
    random.seed(42)

    if spins != "summed":
        raise NotImplementedError("Sorry, only summed spins are implemented right now.")

    if savename:
        fig = plt.figure(figsize=(5, 8))

    Efermi = 0.0
    occupied_up_to = Efermi
    print("Fermi level = ", occupied_up_to)

    if not normalization:
        normalization = 1

    if population_sources == "all":
        tdos_keys = list(tdos.keys())
        non_sources = ["E", "up", "down"]
        population_sources = [k for k in tdos_keys if k not in non_sources]

    for src in population_sources:
        orig = tdos[src]
        to_plot = np.array(orig) / normalization
        tdos[src] = to_plot

    for el in population_sources:
        if el in colors:
            color = color_palette[colors[el]]
        else:
            color = color_palette[random.choice(list(color_palette.keys()))]

        label = el
        if special_labels:
            if el in special_labels:
                label = special_labels[el]

        energies = tdos["E"]
        populations = tdos[el]

        occ_energies = []
        occ_populations = []
        unocc_energies = []
        unocc_populations = []

        for idx, E in enumerate(energies):
            if E == occupied_up_to:
                occ_energies.append(energies[idx])
                occ_populations.append(populations[idx])
                unocc_energies.append(energies[idx])
                unocc_populations.append(populations[idx])
            if E < occupied_up_to:
                occ_energies.append(energies[idx])
                occ_populations.append(populations[idx])
            elif E > occupied_up_to:
                unocc_energies.append(energies[idx])
                unocc_populations.append(populations[idx])

        # smearing with Gaussian filter
        if smearing:
            occ_populations = gaussian_filter1d(occ_populations, smearing)
            unocc_populations = gaussian_filter1d(unocc_populations, smearing)

        ax = plt.plot(
            occ_populations,
            occ_energies,
            color=color,
            label=label,
            alpha=params["line_alpha"],
            lw=params["lw"],
        )
        ax = plt.plot(
            unocc_populations,
            unocc_energies,
            color=color,
            label="__nolegend__",
            alpha=params["line_alpha"],
            lw=params["lw"],
        )
        ax = plt.fill_betweenx(
            occ_energies, occ_populations, color=color, alpha=params["fill_alpha"], lw=0
        )

    ax = plt.axhline(y=Efermi, color="black", linestyle="--")

    ax = plt.xticks(xticks[1])
    ax = plt.yticks(yticks[1])
    if not xticks[0]:
        ax = plt.gca().xaxis.set_ticklabels([])
    if not yticks[0]:
        ax = plt.gca().yaxis.set_ticklabels([])
    ax = plt.xlabel(xlabel)
    ax = plt.ylabel(ylabel)
    ax = plt.title(title)
    ax = plt.xlim(xlim)
    ax = plt.ylim(ylim)

    if legend:
        if isinstance(legend, str):
            ax = plt.legend(loc=legend)
        else:
            ax = plt.legend(loc="best")

    if show:
        plt.show()

    if savename:
        plt.savefig(savename)

    return ax



def ax_tcohp(
    tcohp,
    colors,
    population_sources="all",
    color_palette=get_colors("set2"),
    params={"line_alpha": 0.9, "fill_alpha": 0.3, "lw": 1},
    special_labels=None,
    normalization=None,
    flip = True,
    bond_type = "cohp",
    smearing=2.0,
    Efermi=0.0,
    xlim=(-6, 6),
    ylim=(-2, 2),
    xticks=(True, [-6, -4, -2, 0, 2, 4, 6]),
    yticks=(True, [-2, -1, 0, 1, 2]),   
    xlabel=r"$-COHP$",
    ylabel=r"$E-E_F\/(eV)$",
    legend=True,
    title=None,
    savename=False,
    show=True,
    ):

    """
    Args:
        tcohp (dict)
            result of pydmclab.hpc.analyze.tcohp()
                list(d.keys()) = ['E'] + [el1-el2 (str) for all pairs of elements in structure that might be bonding]
                    note: el1-el2 is sorted alphabetically
                    e.g., the keys for Ca2Ru2O7 might be: ['Ca-Ca', 'Ca-Ru', 'Ca-O', 'Ru-Ru', 'O-Ru', 'O-O', 'E']
                d['E'] = 1d array of energies corresponding with DOS
                for each "bond type" (el1-el2), we have two keys: ['cohp', 'icohp']
                    cohp --> absolute populations
                    icohp --> integrated populations
                    d[el1-el2]['cohp' or 'icohp'] will return a 1d array of populations summed over all interactions of el1-el2 in the structure, summed over all spins
                so if I wanted to plot the total COHP for the interaction between Ru and O throughout the structure:
                    energies = d['E']
                    cohp = d['O-Ru']['cohp']
                    plt.plot(cohp, energies)
        
        colors (dict)
            {bond : color (str)}
            
        population_sources (str or list)
            'all' : plot all bonds
            ['S-S'] : plots just the S-S bonds
            ['S-S', 'P-S'] : plots just S-S and P-S bonds


        color_palette (dict)
            {color (str) : rgb (tuple)}
                usually loaded from pydmc.utils.plotting.get_colors()

        params (dict)
            {'fill_alpha' : transparency for occ populations,
             'line_alpha' : transparency for COHP line,
             'lw' : COHP linewidth}

        special_labels (dict)
            {bond : label (str)}

        normalization (float)
            divide COHP by this number
                common normalizations:
                    1 : no normalization (same as None)
                    CompTools(formula).n_atoms (per formula unit)
                    results['meta']['all_input_parameters']['NELECT'] (per electron)
        
        flip = (bool)
            flip the COHP value to -COHP
        
        bond_type (str)
            "cohp" or "icohp"
        
        smearing (float or False)
            std. dev. for Gaussian smearing of COHP or False for no smearing

        Efermi (float)
            Fermi level (eV)

        xlim (tuple)
            (xmin (float), xmax (float))

        ylim (tuple)
            (ymin (float), ymax (float))

        xticks (tuple)
            (bool to show label or not, (xtick0, xtick1, ...))

        xlabel (str)
            x-axis label

        ylabel (str)
            y-axis label

        legend (bool)
            include legend

        title (str)
            title of plot

        savename (str)
            if False: don't save (just return ax)
            if str, save fig object at this location

        show (bool)
            show plot

    Returns:
        matplotlib axis object
    """
    set_rc_params()
    random.seed(42)

    if savename:
        fig = plt.figure(figsize=(5, 8))

    Efermi = 0.0
    occupied_up_to = Efermi
    print("Fermi level = ", occupied_up_to)

    if not normalization:
        normalization = 1

    if population_sources == "all":
        tcohp_keys = list(tcohp.keys())
        non_sources = ["E"]
        population_sources = [k for k in tcohp_keys if k not in non_sources]

    for src in population_sources:
        orig = tcohp[src][bond_type]
        if not flip:
            to_plot = np.array(orig) / normalization
        else:
            to_plot = -np.array(orig) / normalization
        tcohp[src] = to_plot

    for bond in population_sources:
        if bond in colors:
            color = color_palette[colors[bond]]
        else:
            color = color_palette[random.choice(list(color_palette.keys()))]

        label = bond
        if special_labels:
            if bond in special_labels:
                label = special_labels[bond]

        energies = tcohp['E']
        populations = tcohp[bond]
        
        occ_energies = []
        occ_populations = []
        unocc_energies = []
        unocc_populations = []
        
        for idx, E in enumerate(energies):
            if E == occupied_up_to:
                occ_energies.append(energies[idx])
                occ_populations.append(populations[idx])
                unocc_energies.append(energies[idx])
                unocc_populations.append(populations[idx])
            if E < occupied_up_to:
                occ_energies.append(energies[idx])
                occ_populations.append(populations[idx])
            elif E > occupied_up_to:
                unocc_energies.append(energies[idx])
                unocc_populations.append(populations[idx])
                

        # smearing with Gaussian filter
        if smearing:
            occ_populations = gaussian_filter1d(occ_populations, smearing)
            unocc_populations = gaussian_filter1d(unocc_populations, smearing)

        ax = plt.plot(
            occ_populations,
            occ_energies,
            color=color,
            label=label,
            alpha=params["line_alpha"],
            lw=params["lw"],
        )
        ax = plt.plot(
            unocc_populations,
            unocc_energies,
            color=color,
            label="__nolegend__",
            alpha=params["line_alpha"],
            lw=params["lw"],
        )
        ax = plt.fill_betweenx(
            occ_energies, occ_populations, color=color, alpha=params["fill_alpha"], lw=0
        )
        
    ax = plt.axvline(x=0, color='black', linestyle='--')        
    ax = plt.axhline(y=Efermi, color='black', linestyle='--')

    ax = plt.xticks(xticks[1])
    ax = plt.yticks(yticks[1])
    if not xticks[0]:
        ax = plt.gca().xaxis.set_ticklabels([])      
    if not yticks[0]:
        ax = plt.gca().yaxis.set_ticklabels([])
    ax = plt.xlabel(xlabel)
    ax = plt.ylabel(ylabel)
    ax = plt.title(title)
    ax = plt.xlim(xlim)
    ax = plt.ylim(ylim)   

    if legend:
        if isinstance(legend, str):
            ax = plt.legend(loc=legend)
        else:
            ax = plt.legend(loc="best")

    if show:
        plt.show()

    if savename:
        plt.savefig(savename)
        
    return ax


from pydmclab.core.comp import CompTools
from pydmclab.utils.handy import eVat_to_kJmol, kJmol_to_eVat

from pymatgen.analysis.reaction_calculator import Reaction
from pymatgen.core.composition import Composition


def main():
    formation_energies = {"Li": 0, "NbO2": -2, "LiNbO2": -3}
    reactants = ["Li", "NbO2"]
    products = ["LiNbO2"]

    re = ReactionEnergy(formation_energies, reactants, products)
    return re


if __name__ == "__main__":
    re = main()
